package sun.java2d.d3d;

import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import sun.misc.PerfCounter;
import java.awt.Toolkit;
import java.awt.GraphicsConfiguration;
import java.util.ArrayList;
import java.awt.DisplayMode;
import java.awt.Dialog;
import sun.awt.windows.WWindowPeer;
import java.awt.peer.WindowPeer;
import java.awt.Frame;
import sun.java2d.windows.WindowsFlags;
import java.awt.event.WindowListener;
import java.awt.Window;
import java.awt.Rectangle;
import sun.java2d.pipe.hw.ContextCapabilities;
import sun.awt.Win32GraphicsDevice;

public class D3DGraphicsDevice extends Win32GraphicsDevice
{
    private D3DContext context;
    private static boolean d3dAvailable;
    private ContextCapabilities d3dCaps;
    private boolean fsStatus;
    private Rectangle ownerOrigBounds;
    private boolean ownerWasVisible;
    private Window realFSWindow;
    private WindowListener fsWindowListener;
    private boolean fsWindowWasAlwaysOnTop;
    
    private static native boolean initD3D();
    
    public static D3DGraphicsDevice createDevice(int n) {
        if (!D3DGraphicsDevice.d3dAvailable) {
            return null;
        }
        ContextCapabilities deviceCaps = getDeviceCaps(n);
        if ((deviceCaps.getCaps() & 0x40000) == 0x0) {
            if (WindowsFlags.isD3DVerbose()) {
                System.out.println("Could not enable Direct3D pipeline on screen " + n);
            }
            return null;
        }
        else {
            if (WindowsFlags.isD3DVerbose()) {
                System.out.println("Direct3D pipeline enabled on screen " + n);
            }
            return new D3DGraphicsDevice(n, deviceCaps);
        }
    }
    
    private static native int getDeviceCapsNative(int p0);
    
    private static native String getDeviceIdNative(int p0);
    
    private static ContextCapabilities getDeviceCaps(final int n) {
        D3DContext.D3DContextCaps d3DContextCaps = null;
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            class Result
            {
                int caps;
                String id;
            }
            final D3DGraphicsDevice.1Result result = new Result();
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    result.caps = D3DGraphicsDevice.access$000(n);
                    result.id = D3DGraphicsDevice.access$100(n);
                }
            });
            d3DContextCaps = new D3DContext.D3DContextCaps(result.caps, result.id);
        }
        finally {
            instance.unlock();
        }
        return (d3DContextCaps != null) ? d3DContextCaps : new D3DContext.D3DContextCaps(0, null);
    }
    
    public final boolean isCapPresent(int n) {
        return (this.d3dCaps.getCaps() & n) != 0x0;
    }
    
    private D3DGraphicsDevice(int i, ContextCapabilities d3dCaps) {
        super(i);
        this.ownerOrigBounds = null;
        this.descString = "D3DGraphicsDevice[screen=" + i;
        this.d3dCaps = d3dCaps;
        this.context = new D3DContext(D3DRenderQueue.getInstance(), this);
    }
    
    public boolean isD3DEnabledOnDevice() {
        return this.isValid() && this.isCapPresent(262144);
    }
    
    public static boolean isD3DAvailable() {
        return D3DGraphicsDevice.d3dAvailable;
    }
    
    private Frame getToplevelOwner(Window owner) {
        while (owner != null) {
            owner = owner.getOwner();
            if (!(owner instanceof Frame)) {
                continue;
            }
            return (Frame)owner;
        }
        return null;
    }
    
    private static native boolean enterFullScreenExclusiveNative(int p0, long p1);
    
    protected void enterFullScreenExclusive(final int n, WindowPeer windowPeer) {
        final WWindowPeer wWindowPeer = (WWindowPeer)this.realFSWindow.getPeer();
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    long hWnd = wWindowPeer.getHWnd();
                    if (hWnd == 0L) {
                        D3DGraphicsDevice.this.fsStatus = false;
                        return;
                    }
                    D3DGraphicsDevice.this.fsStatus = D3DGraphicsDevice.access$300(n, hWnd);
                    return;
                }
            });
        }
        finally {
            instance.unlock();
        }
        if (!this.fsStatus) {
            super.enterFullScreenExclusive(n, windowPeer);
        }
    }
    
    private static native boolean exitFullScreenExclusiveNative(int p0);
    
    protected void exitFullScreenExclusive(final int n, WindowPeer windowPeer) {
        if (this.fsStatus) {
            D3DRenderQueue instance = D3DRenderQueue.getInstance();
            instance.lock();
            try {
                instance.flushAndInvokeNow(new Runnable() {
                    @Override
                    public void run() {
                        D3DGraphicsDevice.access$400(n);
                    }
                });
            }
            finally {
                instance.unlock();
            }
        }
        else {
            super.exitFullScreenExclusive(n, windowPeer);
        }
    }
    
    protected void addFSWindowListener(Window realFSWindow) {
        if (!(realFSWindow instanceof Frame) && !(realFSWindow instanceof Dialog) && (this.realFSWindow = this.getToplevelOwner(realFSWindow)) != null) {
            this.ownerOrigBounds = this.realFSWindow.getBounds();
            WWindowPeer wWindowPeer = (WWindowPeer)this.realFSWindow.getPeer();
            this.ownerWasVisible = this.realFSWindow.isVisible();
            Rectangle bounds = realFSWindow.getBounds();
            wWindowPeer.reshape(bounds.x, bounds.y, bounds.width, bounds.height);
            wWindowPeer.setVisible(true);
        }
        else {
            this.realFSWindow = realFSWindow;
        }
        this.fsWindowWasAlwaysOnTop = this.realFSWindow.isAlwaysOnTop();
        ((WWindowPeer)this.realFSWindow.getPeer()).setAlwaysOnTop(true);
        this.fsWindowListener = new D3DGraphicsDevice.D3DFSWindowAdapter();
        this.realFSWindow.addWindowListener(this.fsWindowListener);
    }
    
    protected void removeFSWindowListener(Window window) {
        this.realFSWindow.removeWindowListener(this.fsWindowListener);
        this.fsWindowListener = null;
        WWindowPeer wWindowPeer = (WWindowPeer)this.realFSWindow.getPeer();
        if (wWindowPeer != null) {
            if (this.ownerOrigBounds != null) {
                if (this.ownerOrigBounds.width == 0) {
                    this.ownerOrigBounds.width = 1;
                }
                if (this.ownerOrigBounds.height == 0) {
                    this.ownerOrigBounds.height = 1;
                }
                wWindowPeer.reshape(this.ownerOrigBounds.x, this.ownerOrigBounds.y, this.ownerOrigBounds.width, this.ownerOrigBounds.height);
                if (!this.ownerWasVisible) {
                    wWindowPeer.setVisible(false);
                }
                this.ownerOrigBounds = null;
            }
            if (!this.fsWindowWasAlwaysOnTop) {
                wWindowPeer.setAlwaysOnTop(false);
            }
        }
        this.realFSWindow = null;
    }
    
    private static native DisplayMode getCurrentDisplayModeNative(int p0);
    
    protected DisplayMode getCurrentDisplayMode(final int n) {
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            class Result
            {
                DisplayMode dm;
                
                Result() {
                    this.dm = null;
                }
            }
            final D3DGraphicsDevice.2Result result = new Result();
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    result.dm = D3DGraphicsDevice.access$600(n);
                }
            });
            if (result.dm == null) {
                return super.getCurrentDisplayMode(n);
            }
            return result.dm;
        }
        finally {
            instance.unlock();
        }
    }
    
    private static native void configDisplayModeNative(int p0, long p1, int p2, int p3, int p4, int p5);
    
    protected void configDisplayMode(final int n, WindowPeer windowPeer, final int n2, final int n3, final int n4, final int n5) {
        if (!this.fsStatus) {
            super.configDisplayMode(n, windowPeer, n2, n3, n4, n5);
            return;
        }
        final WWindowPeer wWindowPeer = (WWindowPeer)this.realFSWindow.getPeer();
        if (this.getFullScreenWindow() != this.realFSWindow) {
            Rectangle bounds = this.getDefaultConfiguration().getBounds();
            wWindowPeer.reshape(bounds.x, bounds.y, n2, n3);
        }
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    long hWnd = wWindowPeer.getHWnd();
                    if (hWnd == 0L) {
                        return;
                    }
                    D3DGraphicsDevice.access$700(n, hWnd, n2, n3, n4, n5);
                    return;
                }
            });
        }
        finally {
            instance.unlock();
        }
        return;
    }
    
    private static native void enumDisplayModesNative(int p0, ArrayList p1);
    
    protected void enumDisplayModes(final int n, final ArrayList list) {
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    D3DGraphicsDevice.access$800(n, list);
                }
            });
            if (list.size() == 0) {
                list.add(getCurrentDisplayModeNative(n));
            }
        }
        finally {
            instance.unlock();
        }
    }
    
    private static native long getAvailableAcceleratedMemoryNative(int p0);
    
    public int getAvailableAcceleratedMemory() {
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            class Result
            {
                long mem;
                
                Result() {
                    this.mem = 0L;
                }
            }
            final D3DGraphicsDevice.3Result result = new Result();
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    result.mem = D3DGraphicsDevice.access$900(D3DGraphicsDevice.this.getScreen());
                }
            });
            return (int)result.mem;
        }
        finally {
            instance.unlock();
        }
    }
    
    public GraphicsConfiguration[] getConfigurations() {
        if (this.configs == null && this.isD3DEnabledOnDevice()) {
            this.defaultConfig = this.getDefaultConfiguration();
            if (this.defaultConfig != null) {
                (this.configs = new GraphicsConfiguration[1])[0] = this.defaultConfig;
                return (GraphicsConfiguration[])this.configs.clone();
            }
        }
        return super.getConfigurations();
    }
    
    public GraphicsConfiguration getDefaultConfiguration() {
        if (this.defaultConfig == null) {
            if (this.isD3DEnabledOnDevice()) {
                this.defaultConfig = new D3DGraphicsConfig(this);
            }
            else {
                this.defaultConfig = super.getDefaultConfiguration();
            }
        }
        return this.defaultConfig;
    }
    
    private static native boolean isD3DAvailableOnDeviceNative(int p0);
    
    public static boolean isD3DAvailableOnDevice(final int n) {
        if (!D3DGraphicsDevice.d3dAvailable) {
            return false;
        }
        D3DRenderQueue instance = D3DRenderQueue.getInstance();
        instance.lock();
        try {
            class Result
            {
                boolean avail;
                
                Result() {
                    this.avail = false;
                }
            }
            final D3DGraphicsDevice.4Result result = new Result();
            instance.flushAndInvokeNow(new Runnable() {
                @Override
                public void run() {
                    result.avail = D3DGraphicsDevice.access$1000(n);
                }
            });
            return result.avail;
        }
        finally {
            instance.unlock();
        }
    }
    
    D3DContext getContext() {
        return this.context;
    }
    
    ContextCapabilities getContextCapabilities() {
        return this.d3dCaps;
    }
    
    public void displayChanged() {
        super.displayChanged();
        if (D3DGraphicsDevice.d3dAvailable) {
            this.d3dCaps = getDeviceCaps(this.getScreen());
        }
    }
    
    protected void invalidate(int n) {
        super.invalidate(n);
        this.d3dCaps = new D3DContext.D3DContextCaps(0, null);
    }
    
    static int access$000(int n) {
        return getDeviceCapsNative(n);
    }
    
    static String access$100(int n) {
        return getDeviceIdNative(n);
    }
    
    static boolean access$300(int n, long n2) {
        return enterFullScreenExclusiveNative(n, n2);
    }
    
    static boolean access$400(int n) {
        return exitFullScreenExclusiveNative(n);
    }
    
    static DisplayMode access$600(int n) {
        return getCurrentDisplayModeNative(n);
    }
    
    static void access$700(int n, long n2, int n3, int n4, int n5, int n6) {
        configDisplayModeNative(n, n2, n3, n4, n5, n6);
    }
    
    static void access$800(int n, ArrayList list) {
        enumDisplayModesNative(n, list);
    }
    
    static long access$900(int n) {
        return getAvailableAcceleratedMemoryNative(n);
    }
    
    static boolean access$1000(int n) {
        return isD3DAvailableOnDeviceNative(n);
    }
    
    static {
        Toolkit.getDefaultToolkit();
        D3DGraphicsDevice.d3dAvailable = initD3D();
        if (D3DGraphicsDevice.d3dAvailable) {
            D3DGraphicsDevice.pfDisabled = true;
            PerfCounter.getD3DAvailable().set(1L);
        }
        else {
            PerfCounter.getD3DAvailable().set(0L);
        }
    }
    
    private static class D3DFSWindowAdapter extends WindowAdapter
    {
        @Override
        public void windowDeactivated(WindowEvent windowEvent) {
            D3DRenderQueue.getInstance();
            D3DRenderQueue.restoreDevices();
        }
        
        @Override
        public void windowActivated(WindowEvent windowEvent) {
            D3DRenderQueue.getInstance();
            D3DRenderQueue.restoreDevices();
        }
    }
}
